; eo-acc.el -- minor mode providing keyboard input of 8-bit chars
; for LATIN-3 (ISO-8859-3).

; Copyright (C) 1996, 1998 by Sergei B. Pokrovsky, <pok@iis.nsk.su>

; Made by Sergei B. Pokrovsky from and distributable
; on the same terms as
; ====== start quote
;;; iso-acc.el --- minor mode providing electric accent keys

;; Copyright (C) 1993, 1994, 1996 Free Software Foundation, Inc.

;; Author: Johan Vromans <jv@mh.nl>
;; Keywords: i18n
;; Adapted to XEmacs 19.14 by Alexandre Oliva <oliva@dcc.unicamp.br>
;; $Revision: 1.9 $
;; $Date: 1997/01/31 06:35:24 $

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

; ========= end quote

; This modified program is intended to serve as a replacement for the
; iso-acc.el module supplied with the standard emacs package. To install
; it this eo-acc.el, somewhere on your load path, byte-compile it and add
; (require 'eo-acc) to your init file (.emacs). You may find it convenient
; to bind the eo-accent-mode to a key of your choice in the usual way.

; It is recommended to use it in combination with iso03-syntax.el, which
; would ensure correct case-dependent operation.

;;; Commentary on the program's functions:

;; Function `eo-accents-mode' activates a minor mode in which
;; typewriter "dead keys" are emulated.  The purpose of this emulation
;; is to provide a simple means for inserting accented characters
;; according to the ISO-8859-3 character set (with special attention
;; to the esperantic diacritics).
;;
;; In `eo-accents-mode', pseudo accent characters are used to
;; introduce accented keys.  The pseudo-accent characters are:
;;
;;   %  (percent)   -> grave accent
;;   !  (exclamation mark)  -> acute accent or dot over the character
;;   :  (semicolon) -> diaeresis
;;   ^  (caret)     -> circumflex
;;   ~  (tilde)     -> tilde or breve or stroke through the character
;;   ;  (semicolon) -> esperantic diacritics
;;
;; The action taken depends on the key that follows the pseudo accent.
;; In general: 
;;
;;   pseudo-accent + appropriate letter -> accented letter
;;   pseudo-accent + other -> pseudo-accent + other
;;   
;; If the pseudo-accent is followed by anything else than a 
;; self-insert-command, the dead-key code is terminated, the
;; pseudo-accent inserted `as is'.
;
;  If you need a pseudo-accent character in its literal meaning before
;  an accent-sensitive character, e.g. ;c (and not ), just type
;  the dead-key, space, backspace.
;  
;  Only the letters required by ISO-8859-3 are supported (i.e. nothing
;  is provided for the 7 holes a5, ae, be, c3, d0, e3, f0).
;
;; Function `eo-accents-mode' can be used to enable the eo-accents
;; minor mode, or to disable it.

;; If you want only some of these characters to serve as accents,
;; set eo-accents-enable to the list of characters that should be special.

;  There are two ways to type the Esperanto diacritics: 
;  1) The general method:  irka = ^cirka~u  etc. 
;  2) A special "semicolon method":  irka = ;cirka;u
;     or even irka = ;cirk;a (;a = a)


;;; Code:
(provide 'eo-acc)

;; needed for compatibility with XEmacs 19.14
(if (fboundp 'read-event) ()
  (defun read-event () (event-key (next-command-event))))

(if (fboundp 'character-to-event)
    (defun iso-char-to-event (ch)
      "returns an event containing the given character"
      (character-to-event (list ch)))
  (defun iso-char-to-event (ch)
    "returns the character itself"
    ch))

;; end of compatibility modules

(defvar eo-accents-list
  '(((?% ?A) ?\301)
    ((?% ?E) ?\311)
    ((?% ?I) ?\315)
    ((?% ?O) ?\323)
    ((?% ?U) ?\332)
    ((?! ?Z) ?\257)  ;; \.Z --- pok
    ((?% ?a) ?\341)
    ((?% ?e) ?\351)
    ((?% ?i) ?\355)
    ((?% ?o) ?\363)
    ((?% ?u) ?\372)
    ((?! ?A) ?\300)
    ((?! ?C) ?\305)  ;; \.C --- pok
    ((?! ?E) ?\310)
    ((?! ?G) ?\325)  ;; \.G --- pok
    ((?! ?I) ?\314)
    ((?! ?O) ?\322)
    ((?! ?U) ?\331)
    ((?! ?a) ?\340)
    ((?! ?c) ?\345)  ;; \.c --- pok
    ((?! ?e) ?\350)
    ((?! ?g) ?\365)  ;; \.g --- pok
    ((?! ?i) ?\354)
    ((?! ?o) ?\362)
    ((?! ?u) ?\371)
    ((?! ?z) ?\277)  ;; \.z --- pok
    ((?^ ?A) ?\302)
    ((?^ ?E) ?\312)
    ((?^ ?I) ?\316)
    ((?^ ?O) ?\324)
    ((?^ ?U) ?\333)
    ((?^ ?a) ?\342)
    ((?^ ?e) ?\352)
    ((?^ ?i) ?\356)
    ((?^ ?o) ?\364)
    ((?^ ?u) ?\373)
;------ Regular Esperanto --- sed krom  !!!
    ((?^ ?C) ?\306) ;; ^C --- pok
    ((?^ ?H) ?\246) ;; ^H --- pok
    ((?^ ?J) ?\254) ;; ^J --- pok
    ((?^ ?G) ?\330) ;; ^G --- pok
    ((?^ ?S) ?\336) ;; ^S --- pok
    ((?^ ?U) ?\335) ;; )U --- pok
    ((?^ ?c) ?\346) ;; ^c --- pok
    ((?^ ?h) ?\266) ;; ^h --- pok
    ((?^ ?j) ?\274) ;; ^j --- pok
    ((?^ ?g) ?\370) ;; ^g --- pok
    ((?^ ?s) ?\376) ;; ^s --- pok
;------ tremao kaj germanajxoj ():
    ((?: ?A) ?\304)
    ((?: ?E) ?\313)
    ((?: ?I) ?\317)
    ((?: ?O) ?\326)
    ((?: ?U) ?\334)
    ((?: ?a) ?\344)
    ((?: ?e) ?\353)
    ((?: ?i) ?\357)
    ((?: ?o) ?\366)
    ((?: ?s) ?\337)
    ((?: ?u) ?\374)
;------- tildo kaj turkajxoj ({\u G}, \i) --- pok
    ((?\~ ?C) ?\307)
    ((?\~ ?G) ?\253)  ;; {\u G} --- pok
    ((?\~ ?H) ?\241)  ;; H with stroke --- pok
    ((?\~ ?I) ?\251)  ;; \.I --- pok
    ((?\~ ?N) ?\321)
    ((?\~ ?O) ?\325)
    ((?\~ ?S) ?\252)  ;; ,S --- pok
    ((?\~ ?U) ?\335)  ;; )U --- pok
    ((?\~ ?c) ?\347)
    ((?\~ ?g) ?\273)  ;; {\u g} --- pok
    ((?\~ ?h) ?\261)  ;; h with stroke --- pok
    ((?\~ ?i) ?\271)  ;; \i --- pok
    ((?\~ ?n) ?\361)
    ((?\~ ?o) ?\365)
    ((?\~ ?s) ?\272)  ;; ,s --- pok
    ((?\~ ?u) ?\375)  ;; )u --- pok
; ------------ PUNKTOKOMO:
    ((?\; ?A) ?A)    ;; Au( --- pok
    ((?\; ?C) ?\306) ;; ^C --- pok
    ((?\; ?E) ?E)    ;; Eu( --- pok
    ((?\; ?H) ?\246) ;; ^H --- pok
    ((?\; ?J) ?\254) ;; ^J --- pok
    ((?\; ?G) ?\330) ;; ^G --- pok
    ((?\; ?S) ?\336) ;; ^S --- pok
    ((?\; ?U) ?\335) ;; )U --- pok
    ((?\; ?a) ?a)    ;; au( --- pok
    ((?\; ?c) ?\346) ;; ^c --- pok
    ((?\; ?e) ?e)    ;; eu( --- pok
    ((?\; ?h) ?\266) ;; ^h --- pok
    ((?\; ?j) ?\274) ;; ^j --- pok
    ((?\; ?g) ?\370) ;; ^g --- pok
    ((?\; ?s) ?\376) ;; ^s --- pok
    ((?\; ?u) ?\375) ;; )u --- pok
; ---------- non-alphabetic chars:
    ((?\; ?\_) ?\240) ;; hard space --- pok
    ((?\; ?\~) ?\242) ;; breve --- pok
    ((?\; ?L) ?\243)  ;; pound sterling --- pok
    ((?\; ?\$) ?\244) ;; monetary sign --- pok
    ((?\; ?P) ?\247)  ;; paragrafo (section sign, ; cf ^S!) --- pok
    ((?\; ?:) ?\250)  ;; trema --- pok
    ((?\; ?\-) ?\255) ;; soft hyphen --- pok
    ((?\; ?0) ?\260)  ;; degree sign --- pok
    ((?\; ?2) ?\262)  ;; square sign --- pok
    ((?\; ?3) ?\263)  ;; cube sign --- pok
    ((?\; ?5) ?\275)  ;; 1/2 (=.5) sign --- pok
    ((?\; ?\%) ?\264) ;; minute sign (acute accent) --- pok
    ((?\; ?m) ?\265)  ;; micro sign (mu) --- pok
    ((?\; ?\*) ?\267) ;; central dot --- pok
    ((?\; ?\,) ?\270) ;; cedilla --- pok
    ((?\; ?x) ?\327)  ;; times sign --- pok
    ((?\; ?\/) ?\367) ;; integer div sign --- pok
    ((?\; ?\.) ?\377) ;; dot above --- pok
    )
  "Association list for ISO-8859-3 accent combinations.")

(defvar eo-accents-mode nil
  "*Non-nil enables eo-accents mode.
Setting this variable makes it local to the current buffer.
See function `eo-accents-mode'.")
(make-variable-buffer-local 'eo-accents-mode)

(defun eo-accents-accent-key (prompt)
  "Modify the following character by adding an accent to it."
  ;; Pick up the accent character.
  (if (and eo-accents-mode
	   (memq last-input-char eo-accents-enable))
      (eo-accents-compose prompt)
    (char-to-string last-input-char)))

(defun eo-accents-compose-key (prompt)
  "Modify the following character by adding an accent to it."
  ;; Pick up the accent character.
  (let ((combined (eo-accents-compose prompt)))
    (if unread-command-events
	(let ((unread unread-command-events))
	  (setq unread-command-events nil)
	  (error "Characters %s and %s cannot be composed"
		 (single-key-description (aref combined 0))
		 (single-key-description (car unread)))))
    combined))

(defun eo-accents-compose (prompt)
  (let* ((first-char last-input-char)
	 ;; Wait for the second key and look up the combination.
	 (second-char (if (or prompt
			      (not (eq (key-binding "a")
				       'self-insert-command)))
			  (progn
			    (message "%s%c"
				     (or prompt "Compose with ")
				     first-char)
			    (read-event))
			(insert first-char)
			(prog1 (read-event)
			  (delete-region (1- (point)) (point)))))
	 (entry (assoc (list first-char second-char) eo-accents-list)))
    (if entry
	;; Found it: delete the first character and insert the combination.
       (progn
          (let ((found (nth 1 entry)))
          (if (char-equal found second-char) ; esperantic digraphs -- pok
              (setq unread-command-events
                    (cons (iso-char-to-event ?\375) unread-command-events))
          )
          (vector (iso-char-to-event found))))
      ;; Otherwise, advance and schedule the second key for execution.
      (setq unread-command-events (cons (iso-char-to-event second-char)
					unread-command-events))
      (vector (iso-char-to-event first-char)))))

(defvar eo-accents-enable '(?% ?! ?^ ?: ?~ ?;)
  "*List of accent keys that become prefixes in Eo-Accents mode.
The default is (?% ?! ?^ ?: ?~ ?;), which provides for all the supported
accent keys.  For certain languages, you might want to remove some of
those characters that are not actually used.")

(or key-translation-map (setq key-translation-map (make-sparse-keymap)))
;; For sequences starting with an accent character,
;; use a function that tests eo-accents-mode.
(if (memq ?% eo-accents-enable)
    (define-key key-translation-map "%"  'eo-accents-accent-key))
(if (memq ?! eo-accents-enable)
    (define-key key-translation-map "!"  'eo-accents-accent-key))
(if (memq ?^ eo-accents-enable)
    (define-key key-translation-map "^"  'eo-accents-accent-key))
(if (memq ?: eo-accents-enable)
    (define-key key-translation-map ":"  'eo-accents-accent-key))
(if (memq ?~ eo-accents-enable)
    (define-key key-translation-map "~"  'eo-accents-accent-key))
(if (memq ?; eo-accents-enable)
    (define-key key-translation-map ";"  'eo-accents-accent-key))

;; The minor mode is not well indicated in the mode line (once turned on,
;; it does not disappear when it is turned off). If you dislike this,
;; comment out the next four lines.
 (or (assq 'eo-accents-mode minor-mode-map-alist)
     (setq minor-mode-alist
 	  (append minor-mode-alist
 		  '((eo-accents-mode " 3L")))))


;;;###autoload
(defun eo-accents-mode (&optional arg)
  "Toggle eo-accents mode, in which accents may modify the following char
(EO stands for \"Esperanto\" according to ISO-639).
This permits easy insertion of 8-bit characters from ISO-8859-3,
aka Latin-3.
When eo-accents mode is enabled, some pseudo-accent character keys
do not self-insert; instead, they modify the following letter key so that
it inserts a Latin-3 8-bit character. The pseudo-accent characters are:
  %  (percent)   -> acute accent
  !  (excl.mark) -> grave accent or dot over the character
  :  (semicolon) -> diaeresis
  ^  (caret)     -> circumflex
  ~  (tilde)     -> tilde or breve or stroke through the character
  ;  (semicolon) -> esperantic diacritics
If you need n occurrences of a pseudo-accent character in its literal
meaning before an accent-sensitive character, just type n+1 times the
dead-key, backspace, the sensitive character.
  
The variable `eo-accents-enable' specifies the list of characters to
enable as accents.  If you don't need all of them, remove the ones you
don't need from that list.

Special combinations: 
~c and ~s --- gives c, s with cedilla;
~i, ~g    --- are the Turkish {\i}, {\u g};
~h        --- is the Maltese h with stroke;
:s        --- gives German sharp s;
;a ;c, ;e ;g, ;h, ;j, ;s, ;u
          --- are the esperantic letters;
!         --- acts like dot_above over consonants,
              as grave accent over vocals (also for !i, cf ~i)."


; With an argument, a positive argument enables Eo-Accents-Mode, 
; and a negative argument disables it.

  (interactive "P")

  (if (if arg
	  ;; Negative arg means switch it off.
	  (<= (prefix-numeric-value arg) 0)
	;; No arg means toggle.
      eo-accents-mode)
      (setq eo-accents-mode nil
;           vm-mime-8bit-composition-charset "iso-8859-3"
      )
    ;; Enable electric accents.
    (setq eo-accents-mode t)))

;;; Use Eo-Accents mode in the minibuffer
;;; if it was in use in the previous buffer.
;
;(defun eo-acc-minibuf-setup ()
;  (setq eo-accents-mode
;        (save-excursion
;          (set-buffer (window-buffer minibuffer-scroll-window))
;          eo-accents-mode)))
;
;(add-hook 'minibuf-setup-hook 'eo-acc-minibuf-setup)

;;; eo-acc.el ends here
